import greenfoot.World;
import greenfoot.GreenfootImage;

/**
 * CLASS: Scroll
 * AUTHOR: danpost (greenfoot.org username)
 * CREATED: March 9, 2015
 * MODIFIED: March 14, 2015 (added support for cellsizes greater than one)
 * 
 * This support class controls the background image of the world provided;
 * the size of the image supplied determines the scrolling area.
 * 
 * In your world class, create an instance of this class, supplying the background image of
 * the scrolling area and the world it belongs to, and keep a reference to the Scroll object
 * created in the world.  Then you can use one of the 'changeScroll' methods below to control
 * the scrolling of the background of your active world.
 */
public class Scroll
{
    private int offX, offY; // the scrolling offsets
    private World w; // the world whose background will scroll
    private int wW; // the width of the world (in pixels)
    private int wH; // the height of the world (in pixels)
    private int wC; // the cell-size of the world
    private GreenfootImage bg; // the image for the scrolling background
    private int bgW; // the width of the background image
    private int bgH; // the height of the background image
    
    /** creates a new Scroll object */
    public Scroll(GreenfootImage bgImg, World world)
    {
        // save field values
        bg = bgImg;
        bgW = bg.getWidth();
        bgH = bg. getHeight();
        w = world;
        wW = w.getWidth()*w.getCellSize();
        wH = w.getHeight()*w.getCellSize();
        wC = w.getCellSize();
        // set initial background of world
        w.getBackground().drawImage(bg, 0, 0);
    }
    
    /** use for horizontal scrolling only; returns the change in scroll value */
    public int changeScrollX(int ds)
    {
        // convert cell change to pixel change
        ds *= wC;
        // keep scrolling within image limits
        if (offX+ds >  bgW-wW) ds = bgW-wW-offX;
        if (offX+ds < 0) ds = -offX;
        // perform scrolling
        offX += ds;
        w.getBackground().drawImage(bg, -offX, 0);
        return ds;
    }
    
    /** use for vertical scrolling only; returns the change in scroll value */
    public int changeScrollY(int ds)
    {
        // convert cell change to pixel change
        ds *= wC;
        // keep scrolling within image limits
        if (offY+ds > bgH-wH) ds = bgH-wH-offY;
        if (offY+ds < 0) ds = -offY;
        // perform scrolling
        offY += ds;
        w.getBackground().drawImage(bg, 0, -offY);
        return ds;
    }
    
    /** use for scrolling both horizontal and vertical; returns the changes in scroll values */
    public int[] changeScroll(int dsX, int dsY)
    {
        // convert cell change to pixel change
        dsX *= wC;
        dsY *= wC;
        // keep scrolling within image limits
        if (offX+dsX > bgW-wW) dsX = bgW-wW-offX;
        if (offY+dsY > bgH-wH) dsY = bgH-wH-offY;
        if (offX+dsX < 0) dsX = -offX;
        if (offY+dsY < 0) dsY = -offY;
        // perform scrolling
        offX += dsX;
        offY += dsY;
        w.getBackground().drawImage(bg, -offX, -offY);
        return new int[] { dsX, dsY };
    }
}